package mactrans.system2;

/**
 * MachineTranslation UI (Graphical User Interface) using Swing
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.1 $
 * 
 * <pre>
 * $Log: MachineTranslationGUI.java,v $
 * Revision 1.1  2005/05/25 12:54:41  jkieneck
 * + added to repository
 *
 * </pre>
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import java.util.HashMap;

public class MachineTranslationGUI implements MachineTranslationGUInterface {
	public TextField textInput, textOutput;
	public TextArea textDebug;
	private MachineTranslationSystemInterface mti;
	private JButton btQuit, btTranslate, btClear;
	private JComboBox selAlgorithm, selSource, selTarget, selOutput;
	private JFrame uiFrame;
	private HashMap vConsole;
	
	public void doLogInto( String into, String module, String type, String message ) {
		String current = new String();
		if ( vConsole.containsKey( into ) ) {
			current = (String) vConsole.get( into );
		} else {
			selOutput.addItem( into );
		}

		String line = "[" + module + "]  " + message + "\n";
		current += line;
		vConsole.put( into, current );

		if ( ( (String) selOutput.getSelectedItem() ).equals( into ) ) {
			textDebug.append( line );
		}
	}

	public void doLog( String a, String m, String t ) {
		doLogInto( "all", m, a, t );
		doLogInto( m, m, a, t );
	}

	/** mt
	 * Show output (translation of input sentence)
	 */
	public void writeOutput( String sentence ) 
	{
		textOutput.setText( sentence );
	}
	
	/**
	 * Output (less important) information for the programmer
	 * 
	 * @param module the modulename where the message comes from
	 * @param message the message produced by the module
	 */
	public void debug( String module, String message ) {
		doLog( "", module, message );
	}

	/**
	 * Output warnings for the programmer/user
	 * 
	 * @param module the modulename where the message comes from
	 * @param message the message produced by the module
	 */
	public void warn( String module, String message ) {
		doLog( "warning", module, message );
	}

	/**
	 * Produce an error. Continuation could be tried afterwards
	 * 
	 * @param module the modulename where the message comes from
	 * @param message the message produced by the module
	 */
	public void error( String module, String message ) {
		doLog( "error", module, message );
	}

	/**
	 * Produce a fatal error. Continuation should not be tried afterwards.
	 * 
	 * @param module the modulename where the message comes from
	 * @param message the message produced by the module
	 */
	public void fatalError( String module, String message ) {
		doLog( "fatal", module, message );
	}
	
	public MachineTranslationGUI( MachineTranslationSystemInterface mti ) {
		this.mti = mti;
		initUI();
		show();
		// setup our vConsole output
		vConsole = new HashMap();
	}
	
	public void show()
	{
		uiFrame.setSize(700, 500);
		uiFrame.setVisible(true);
	}	
	
	public void initUI() {
		// Layout
		uiFrame = new JFrame("MachineTranslation");
		uiFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		uiFrame.getContentPane().setLayout(new BorderLayout());
		
		uiFrame.getContentPane().add( topPanel(), BorderLayout.NORTH );
		uiFrame.getContentPane().add( bottomPanel(), BorderLayout.SOUTH );	
		uiFrame.getContentPane().add( new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, leftPanel(), rightPanel() ), BorderLayout.CENTER );
		
		// Interaction: buttons
		btQuit.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				System.exit( 1 );
			}
		} );
		
		btClear.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				textInput.setText( "" );
				textOutput.setText( "" );
			}
		} );
		
		btTranslate.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				mti.setSourceLanguage( (String) selSource.getSelectedItem() );
				mti.setTargetLanguage( (String) selTarget.getSelectedItem() );
				mti.setInputPhrase( textInput.getText() );
				mti.setAlgorithm( (String) selAlgorithm.getSelectedItem() );
				mti.translate();
				//textOutput.setText( mti.getOutputPhrase() );
			}
		} );
		
		selOutput.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				textDebug.setText( (String) vConsole.get( (String) selOutput.getSelectedItem() ) );
			}
		} );
		
		// Interaction: set boxes
		for ( int i = 0; i < mti.getSourceLanguages().length; i++ )  {
			selSource.addItem( mti.getSourceLanguages()[ i ] ); 
		}
		for ( int i = 0; i < mti.getTargetLanguages().length; i++ )  {
			selTarget.addItem( mti.getTargetLanguages()[ i ] ); 
		}
		for ( int i = 0; i < mti.getAlgorithms().length; i++ )  {
			selAlgorithm.addItem( mti.getAlgorithms()[ i ] ); 
		}
	}
	
	protected JPanel topPanel() {
		JPanel topPanel = new JPanel();
		JLabel label = new JLabel( "MachineTranslation", JLabel.CENTER );
		Font font = label.getFont().deriveFont( Font.BOLD + Font.ITALIC, 28 );
		label.setFont( font );
		label.setForeground( Color.black );
		topPanel.setLayout( new BorderLayout() );
		topPanel.add( label, BorderLayout.CENTER );
		return topPanel;
	}
	
	protected JPanel bottomPanel() {
		JPanel result = new JPanel();
		btQuit = new JButton( "Quit" );
		btClear = new JButton( "Reset" );
		btTranslate = new JButton( "Translate" );
		
		result.add( btQuit );
		result.add( btTranslate );
		result.add( btClear );
		return result;
	}
	
	protected JSplitPane rightPanel() {
		textInput = new TextField( 80 );
		textOutput = new TextField( 80 );
		JSplitPane a = new JSplitPane( JSplitPane.VERTICAL_SPLIT, textInput, textOutput );
		textDebug = new TextArea("", 60, 20 );
		/*Font font = textDebug.getFont().deriveFont( Font.BOLD, 10 );
		textDebug.setFont( font );
		textDebug.setForeground( Color.white );
//		textDebug.setBackground( new Color( 0, 0, 128 ) );*/
		return new JSplitPane( JSplitPane.VERTICAL_SPLIT, a, textDebug );
	}
	
	protected JPanel leftPanel() {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		result.add( new JLabel("Source language:"), new GridBagConstraints( 0, 0, 2, 1, 1.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		selSource = new JComboBox();
		selSource.setMinimumSize( new Dimension( 200, 30 ) );
		result.add( selSource, new GridBagConstraints( 1, 1, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		
		result.add( new JLabel("Target language:"), new GridBagConstraints( 0, 2, 2, 1, 1.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		selTarget = new JComboBox();
		selTarget.setMinimumSize( new Dimension( 200, 30 ) );
		result.add( selTarget, new GridBagConstraints( 1, 3, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );

		result.add( new JLabel("Algorithm:"), new GridBagConstraints( 0, 4, 2, 1, 1.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		selAlgorithm = new JComboBox();
		selAlgorithm.setMinimumSize( new Dimension( 200, 30 ) );
		result.add( selAlgorithm, new GridBagConstraints( 1, 5, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );

		result.add( new JLabel("DebugOutput:"), new GridBagConstraints( 0, 6, 2, 1, 1.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		selOutput = new JComboBox();
		selOutput.setMinimumSize( new Dimension( 200, 30 ) );
		result.add( selOutput, new GridBagConstraints( 1, 7, 2, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0 ) );
		
		return result;
	}
	
}
